@@ -264,38 +264,210 @@ xlsx_find_predefined_fill (GnmStyle const *style)
 /****************************************************************************/
 
 static void
-xlsx_write_shared_strings (XLSXWriteState *state, GsfOutfile *wb_part)
+xlsx_write_rich_text (GsfXMLOut *xml, char const *text, PangoAttrList *attrs,
+		      gboolean allow_xml_space)
 {
-	if (state->shared_string_array->len > 0) {
-		unsigned i;
-		GOString const *str;
-		GsfOutput *part = gsf_outfile_open_pkg_add_rel (state->xl_dir, "sharedStrings.xml",
-			"application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml",
-			wb_part,
-			"http://schemas.openxmlformats.org/officeDocument/2006/relationships/sharedStrings");
-		GsfXMLOut *xml = gsf_xml_out_new (part);
+	PangoAttrIterator *iter;
+	int start, end, max;
 
-		gsf_xml_out_start_element (xml, "sst");
-		gsf_xml_out_add_cstr_unchecked (xml, "xmlns", ns_ss);
-		/* Note the schema does not allow the attribute xml:space */
-		gsf_xml_out_add_int (xml, "uniqueCount", state->shared_string_array->len);
-		gsf_xml_out_add_int (xml, "count", state->shared_string_array->len);
+	if (attrs == NULL) {
+		gsf_xml_out_start_element (xml, "t");
+		gsf_xml_out_add_cstr (xml, NULL, text);
+		gsf_xml_out_end_element (xml); /* </t> */
+		return;
+	}
 
-		for (i = 0 ; i < state->shared_string_array->len ; i++) {
-			gsf_xml_out_start_element (xml, "si");
-			gsf_xml_out_start_element (xml, "t");
-			str = g_ptr_array_index (state->shared_string_array, i);
-			gsf_xml_out_add_cstr (xml, NULL, str->str);
-			gsf_xml_out_end_element (xml); /* </t> */
-			gsf_xml_out_end_element (xml); /* </si> */
+	max = strlen (text);
+	iter = pango_attr_list_get_iterator (attrs);
+	do {
+		PangoAttribute *attr;
+		GOFontScript fs = GO_FONT_SCRIPT_STANDARD;
+
+		gsf_xml_out_start_element (xml, "r");
+		gsf_xml_out_start_element (xml, "rPr");
+
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_FAMILY);
+		if (attr) {
+			gsf_xml_out_start_element (xml, "rFont");
+			gsf_xml_out_add_cstr_unchecked (xml, "val", ((PangoAttrString *) attr)->value);
+			gsf_xml_out_end_element (xml); /* </rFont> */
 		}
 
-		gsf_xml_out_end_element (xml); /* </sst> */
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_WEIGHT);
+		if (attr) {
+			gsf_xml_out_start_element (xml, "b");
+			gsf_xml_out_add_cstr_unchecked (xml, "val", ((PangoAttrInt *) attr)->value > PANGO_WEIGHT_NORMAL ? "true": "false");
+			gsf_xml_out_end_element (xml); /* </b> */
+		}
 
-		g_object_unref (xml);
-		gsf_output_close (part);
-		g_object_unref (part);
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_STYLE);
+		if (attr) {
+			gsf_xml_out_start_element (xml, "i");
+			gsf_xml_out_add_cstr_unchecked (xml, "val", ((PangoAttrInt *) attr)->value != PANGO_STYLE_NORMAL ? "true": "false");
+			gsf_xml_out_end_element (xml); /* </i> */
+		}
+
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_STRIKETHROUGH);
+		if (attr) {
+			gsf_xml_out_start_element (xml, "strike");
+			gsf_xml_out_add_cstr_unchecked (xml, "val", ((PangoAttrInt *) attr)->value ? "true": "false");
+			gsf_xml_out_end_element (xml); /* </strike> */
+		}
+
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_FOREGROUND);
+		if (attr) {
+			PangoColor *color = &((PangoAttrColor *) attr)->color;
+			char *buf = g_strdup_printf("ff%02x%02x%02x", color->red >> 8, color->green >> 8, color->blue >> 8);
+			gsf_xml_out_start_element (xml, "color");
+			gsf_xml_out_add_cstr_unchecked (xml, "rgb", buf);
+			gsf_xml_out_end_element (xml); /* </color> */
+			g_free (buf);
+		}
+
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_SIZE);
+		if (attr) {
+			gsf_xml_out_start_element (xml, "sz");
+			gsf_xml_out_add_uint (xml, "val", ((PangoAttrInt *) attr)->value / PANGO_SCALE);
+			gsf_xml_out_end_element (xml); /* </sz> */
+		}
+
+		attr = pango_attr_iterator_get (iter, PANGO_ATTR_UNDERLINE);
+		if (attr) {
+			PangoUnderline u = ((PangoAttrInt *) attr)->value;
+			gsf_xml_out_start_element (xml, "u");
+			switch (u) {
+			case PANGO_UNDERLINE_NONE:
+			default:
+				gsf_xml_out_add_cstr_unchecked (xml, "val", "none");
+				break;
+			case PANGO_UNDERLINE_ERROR:
+				/* not supported by OpenXML */
+				/* Fall through. */
+			case PANGO_UNDERLINE_SINGLE:
+				gsf_xml_out_add_cstr_unchecked (xml, "val", "single");
+				break;
+			case PANGO_UNDERLINE_DOUBLE:
+				gsf_xml_out_add_cstr_unchecked (xml, "val", "double");
+				break;
+			case PANGO_UNDERLINE_LOW:
+				gsf_xml_out_add_cstr_unchecked (xml, "val", "singleAccounting");
+				break;
+			}
+			gsf_xml_out_end_element (xml); /* </u> */
+		}
+
+		attr = pango_attr_iterator_get (iter, go_pango_attr_superscript_get_attr_type ());
+		if (attr && ((PangoAttrInt *) attr)->value)
+			fs = GO_FONT_SCRIPT_SUPER;
+		attr = pango_attr_iterator_get (iter, go_pango_attr_subscript_get_attr_type ());
+		if (attr && ((PangoAttrInt *) attr)->value)
+			fs = GO_FONT_SCRIPT_SUB;
+		if (fs != GO_FONT_SCRIPT_STANDARD) {
+			const char *va = (fs == GO_FONT_SCRIPT_SUB)
+				? "subscript"
+				: "superscript";
+			gsf_xml_out_start_element (xml, "vertAlign");
+			gsf_xml_out_add_cstr_unchecked (xml, "val", va);
+			gsf_xml_out_end_element (xml); /* </vertAlign> */
+		}
+
+		gsf_xml_out_end_element (xml); /* </rPr> */
+		gsf_xml_out_start_element (xml, "t");
+		pango_attr_iterator_range (iter, &start, &end);
+		if (end > max)
+		    end = max;
+		if (start < end) {
+			char *buf = g_strndup (text + start, end - start);
+			if (allow_xml_space) {
+				const char *p;
+				gboolean has_space = FALSE;
+				for (p = buf; *p; p = g_utf8_next_char (p)) {
+					if (g_unichar_isspace (g_utf8_get_char (p))) {
+						has_space = TRUE;
+						break;
+					}
+				}
+				if (has_space)
+					gsf_xml_out_add_cstr_unchecked
+						(xml, "xml:space", "preserve");
+			}
+			gsf_xml_out_add_cstr (xml, NULL, buf);
+			g_free (buf);
+		}
+		gsf_xml_out_end_element (xml); /* </t> */
+		gsf_xml_out_end_element (xml); /* </r> */
+	} while (pango_attr_iterator_next (iter));
+	pango_attr_iterator_destroy (iter);
+}
+
+static int
+xlsx_shared_string (XLSXWriteState *state, GnmValue const *v)
+{
+	gpointer tmp;
+	int i;
+
+	g_return_val_if_fail (VALUE_IS_STRING (v), -1);
+
+	if (g_hash_table_lookup_extended (state->shared_string_hash,
+					  v, NULL, &tmp))
+		i = GPOINTER_TO_INT (tmp);
+	else {
+		GnmValue *v2 = value_dup (v);
+
+		if (VALUE_FMT (v2) && !go_format_is_markup (VALUE_FMT (v2)))
+			value_set_fmt (v2, NULL);
+
+		i = state->shared_string_array->len;
+		g_ptr_array_add (state->shared_string_array, v2);
+		g_hash_table_insert (state->shared_string_hash,
+				     v2, GINT_TO_POINTER (i));
 	}
+
+	return i;
+}
+
+static void
+xlsx_write_shared_strings (XLSXWriteState *state, GsfOutfile *wb_part)
+{
+	const unsigned N = state->shared_string_array->len;
+	unsigned i;
+	GsfOutput *part;
+	GsfXMLOut *xml;
+
+	if (N == 0)
+		return;
+
+	part = gsf_outfile_open_pkg_add_rel
+		(state->xl_dir, "sharedStrings.xml",
+		 "application/vnd.openxmlformats-officedocument.spreadsheetml.sharedStrings+xml",
+		 wb_part,
+		 "http://schemas.openxmlformats.org/officeDocument/2006/relationships/sharedStrings");
+	xml = gsf_xml_out_new (part);
+
+	gsf_xml_out_start_element (xml, "sst");
+	gsf_xml_out_add_cstr_unchecked (xml, "xmlns", ns_ss);
+	gsf_xml_out_add_int (xml, "uniqueCount", N);
+	gsf_xml_out_add_int (xml, "count", N);
+
+	for (i = 0; i < N; i++) {
+		GnmValue const *val =
+			g_ptr_array_index (state->shared_string_array, i);
+		PangoAttrList *attrs = VALUE_FMT (val)
+			? (PangoAttrList *)go_format_get_markup (VALUE_FMT (val))
+			: NULL;
+		gsf_xml_out_start_element (xml, "si");
+		xlsx_write_rich_text (xml,
+				      value_peek_string (val),
+				      attrs,
+				      FALSE);
+		gsf_xml_out_end_element (xml); /* </si> */
+	}
+
+	gsf_xml_out_end_element (xml); /* </sst> */
+
+	g_object_unref (xml);
+	gsf_output_close (part);
+	g_object_unref (part);
 }
 
 static void
@@ -1261,6 +1433,7 @@ xlsx_get_style_id (XLSXWriteState *state, GnmStyle const *style)
 	if (NULL == (tmp = g_hash_table_lookup (state->styles_hash, style))) {
 		g_ptr_array_add (state->styles_array, (gpointer) style);
 		tmp = GINT_TO_POINTER (state->styles_array->len);
+		gnm_style_ref (style);
 		g_hash_table_insert (state->styles_hash, (gpointer) style, tmp);
 	}
 	return GPOINTER_TO_INT (tmp) - 1;
@@ -1306,8 +1479,6 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 	GnmParsePos pp;
 	GnmExprTop const *texpr;
 	GnmExprArrayCorner const *array;
-	GnmValue const *val;
-	gpointer tmp;
 	char *cheesy_span = g_strdup_printf ("%d:%d", extent->start.col+1, extent->end.col+1);
 	Sheet *sheet = (Sheet *)state->sheet;
 	GPtrArray *all_cells = sheet_cells (sheet, extent);
@@ -1378,24 +1549,37 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 
 		for (c = extent->start.col ; c <= extent->end.col ; c++) {
 			GnmCell *cell;
+			GnmValue const *val;
 			GnmStyle const *style;
+			GnmStyle *style1 = NULL;
 			gint style_id;
+			GOFormat const *fmt1, *fmt2;
 
 			cell = g_ptr_array_index (all_cells, cno);
-			if (cell && cell->pos.row == r && cell->pos.col == c)
+			if (cell && cell->pos.row == r && cell->pos.col == c) {
 				cno++;
-			else
+				val = cell->value;
+			} else {
 				cell = NULL;
+				val = NULL;
+			}
 
 			/* FIXME: Use sheet_style_get_row once per row */
 			style = sheet_style_get (sheet, c, r);
+			fmt1 = gnm_style_get_format (style);
+			fmt2 = cell ? gnm_cell_get_format_given_style (cell, style) : fmt1;
+			if (fmt1 != fmt2 && !go_format_is_markup (fmt2)) {
+				style = style1 = gnm_style_dup (style);
+				gnm_style_set_format (style1, fmt2);
+			}
 			style_id = style && style != col_styles[c]
 				? xlsx_get_style_id (state, style)
 				: -1;
+			if (style1)
+				gnm_style_unref (style1);
 
 			if (cell) {
 				xlsx_write_init_row (&needs_row, xml, r, cheesy_span);
-				val = cell->value;
 				gsf_xml_out_start_element (xml, "c");
 				gsf_xml_out_add_cstr_unchecked (xml, "r",
 					cell_coord_name (c, r));
@@ -1420,12 +1604,7 @@ xlsx_write_cells (XLSXWriteState *state, GsfXMLOut *xml,
 					/* A reasonable approximation of * 'is_shared'.  It can get spoofed by
 					 * rich text references to a base * string */
 					if (go_string_get_ref_count (val->v_str.val) > 1) {
-						if (NULL == (tmp = g_hash_table_lookup (state->shared_string_hash, val->v_str.val))) {
-							tmp = GINT_TO_POINTER (state->shared_string_array->len);
-							g_ptr_array_add (state->shared_string_array, val->v_str.val);
-							g_hash_table_insert (state->shared_string_hash, val->v_str.val, tmp);
-						}
-						str_id = GPOINTER_TO_INT (tmp);
+						str_id = xlsx_shared_string (state, val);
 						type = "s";
 					} else
 						type = "str";
@@ -2401,93 +2580,6 @@ xlsx_write_print_info (XLSXWriteState *state, GsfXMLOut *xml)
 
 /**********************************************************************/
 
-static void
-xlsx_write_rich_text (GsfXMLOut *xml, char const *text, PangoAttrList *attrs)
-{
-	PangoAttrIterator *iter;
-	PangoAttribute *attr;
-	int start, end, max;
-	char *buf;
-
-	if (attrs == NULL) {
-		gsf_xml_out_start_element (xml, "t");
-		gsf_xml_out_add_cstr (xml, NULL, text);
-		gsf_xml_out_end_element (xml); /* </t> */
-		return;
-	}
-
-	max = strlen (text);
-	iter = pango_attr_list_get_iterator (attrs);
-	do {
-		gsf_xml_out_start_element (xml, "r");
-		gsf_xml_out_start_element (xml, "rPr");
-		gsf_xml_out_start_element (xml, "rFont");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_FAMILY);
-		gsf_xml_out_add_cstr_unchecked (xml, "val", (attr)? ((PangoAttrString *) attr)->value: "Calibri");
-		gsf_xml_out_end_element (xml); /* </rFont> */
-		gsf_xml_out_start_element (xml, "b");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_WEIGHT);
-		gsf_xml_out_add_cstr_unchecked (xml, "val", (attr && ((PangoAttrInt *) attr)->value > PANGO_WEIGHT_NORMAL)? "true": "false");
-		gsf_xml_out_end_element (xml); /* </b> */
-		gsf_xml_out_start_element (xml, "i");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_STYLE);
-		gsf_xml_out_add_cstr_unchecked (xml, "val", (attr && ((PangoAttrInt *) attr)->value != PANGO_STYLE_NORMAL)? "true": "false");
-		gsf_xml_out_end_element (xml); /* </i> */
-		gsf_xml_out_start_element (xml, "strike");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_STRIKETHROUGH);
-		gsf_xml_out_add_cstr_unchecked (xml, "val", (attr && ((PangoAttrInt *) attr)->value)? "true": "false");
-		gsf_xml_out_end_element (xml); /* </strike> */
-		gsf_xml_out_start_element (xml, "color");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_FOREGROUND);
-		if (attr) {
-			PangoColor *color = &((PangoAttrColor *) attr)->color;
-			buf = g_strdup_printf("FF%2x%2x%2x", color->red >> 8, color->green >> 8, color->blue >> 8);
-			gsf_xml_out_add_cstr_unchecked (xml, "rgb", buf);
-			g_free (buf);
-		} else
-			gsf_xml_out_add_cstr_unchecked (xml, "rgb", "FF000000");
-		gsf_xml_out_end_element (xml); /* </color> */
-		gsf_xml_out_start_element (xml, "sz");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_SIZE);
-		gsf_xml_out_add_uint (xml, "val", (attr)? ((PangoAttrInt *) attr)->value / PANGO_SCALE: 8);
-		gsf_xml_out_end_element (xml); /* </sz> */
-		gsf_xml_out_start_element (xml, "u");
-		attr = pango_attr_iterator_get (iter, PANGO_ATTR_UNDERLINE);
-		if (attr) {
-			PangoUnderline u = ((PangoAttrInt *) attr)->value;
-			switch (u) {
-			case PANGO_UNDERLINE_NONE:
-			default:
-				gsf_xml_out_add_cstr_unchecked (xml, "val", "none");
-				break;
-			case PANGO_UNDERLINE_ERROR: /* not supported by OpenXML */
-			case PANGO_UNDERLINE_SINGLE:
-				gsf_xml_out_add_cstr_unchecked (xml, "val", "single");
-			case PANGO_UNDERLINE_DOUBLE:
-				gsf_xml_out_add_cstr_unchecked (xml, "val", "double");
-			case PANGO_UNDERLINE_LOW:
-				gsf_xml_out_add_cstr_unchecked (xml, "val", "singleAccounting");
-			}
-		} else
-			gsf_xml_out_add_cstr_unchecked (xml, "val", "none");
-		gsf_xml_out_end_element (xml); /* </u> */
-		gsf_xml_out_end_element (xml); /* </rPr> */
-		gsf_xml_out_start_element (xml, "t");
-		gsf_xml_out_add_cstr_unchecked (xml, "xml:space", "preserve");
-		pango_attr_iterator_range (iter, &start, &end);
-		if (end > max)
-		    end = max;
-		if (start > end)
-			start = end;
-		buf = g_strndup (text + start, end - start);
-		gsf_xml_out_add_cstr (xml, NULL, buf);
-		g_free (buf);
-		gsf_xml_out_end_element (xml); /* </t> */
-		gsf_xml_out_end_element (xml); /* </r> */
-	} while (pango_attr_iterator_next (iter));
-	pango_attr_iterator_destroy (iter);
-}
-
 static void
 write_comment_author (gpointer key, G_GNUC_UNUSED gpointer value, GsfXMLOut *xml)
 {
@@ -2541,7 +2633,7 @@ xlsx_write_comments (XLSXWriteState *state, GsfOutput *sheet_part, GSList *objec
 		/* Save text as rich text */
 		g_object_get (ptr->data, "text", &name, "markup", &attrs, NULL);
 		if (name && *name)
-			xlsx_write_rich_text (xml, name, attrs);
+			xlsx_write_rich_text (xml, name, attrs, TRUE);
 		g_free (name);
 		pango_attr_list_unref (attrs);
 		gsf_xml_out_end_element (xml); /* </text> */
@@ -2767,6 +2859,19 @@ xlsx_write_calcPR (XLSXWriteState *state, GsfXMLOut *xml)
 
 #include "xlsx-write-docprops.c"
 
+static gboolean
+rich_value_equal (GnmValue const *a, GnmValue const *b)
+{
+	return value_equal (a, b) &&
+		go_format_eq (VALUE_FMT (a), VALUE_FMT (b));
+}
+
+static guint
+rich_value_hash (GnmValue const *v)
+{
+	return value_hash (v) ^ GPOINTER_TO_UINT (VALUE_FMT (v));
+}
+
 static void
 xlsx_write_workbook (XLSXWriteState *state, GsfOutfile *root_part)
 {
@@ -2783,9 +2888,13 @@ xlsx_write_workbook (XLSXWriteState *state, GsfOutfile *root_part)
 	GnmStyle *style = gnm_style_new_default ();
 
 	state->xl_dir = xl_dir;
-	state->shared_string_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	state->shared_string_hash = g_hash_table_new_full
+		((GHashFunc)rich_value_hash, (GEqualFunc)rich_value_equal,
+		 (GDestroyNotify)value_release, NULL);
 	state->shared_string_array = g_ptr_array_new ();
-	state->styles_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
+	state->styles_hash = g_hash_table_new_full
+		(gnm_style_hash, (GEqualFunc)gnm_style_equal,
+		 (GDestroyNotify)gnm_style_unref, NULL);
 	state->styles_array = g_ptr_array_new ();
 	state->dxfs_hash = g_hash_table_new (g_direct_hash, g_direct_equal);
 	state->dxfs_array = g_ptr_array_new ();
