@@ -792,17 +792,6 @@ gee_create_tooltip (GnmExprEntry *gee, gchar const *str,
 	gtk_style_context_add_class (gtk_widget_get_style_context (label),
 				     "function-help");
 
-#if !GTK_CHECK_VERSION(3,4,0)
-	/* Workaround for theme bugs and missing "inherit" in pre GTK 3.4. */
-	{
-		GtkStyleContext *context = gtk_widget_get_style_context (tip);
-		GtkStateFlags state = GTK_STATE_FLAG_NORMAL;
-		GdkRGBA c;
-		gtk_style_context_get_color (context, state, &c);
-		gtk_widget_override_color (label, state, &c);
-	}
-#endif
-
 	if (str)
 		markup = gnm_func_convert_markup_to_pango (str, label);
 	string = g_string_new (markup);
@@ -1703,10 +1692,8 @@ gee_class_init (GObjectClass *gobject_class)
 		G_TYPE_NONE, 0);
 
 
-	/* FIXME: remove the test when we require gtk+ 2.20 or later */
-	if (g_object_interface_find_property (g_type_interface_peek (gobject_class, gtk_cell_editable_get_type ()), "editing-canceled"))
-		g_object_class_override_property
-			(gobject_class, PROP_EDITING_CANCELED, "editing-canceled");
+	g_object_class_override_property
+		(gobject_class, PROP_EDITING_CANCELED, "editing-canceled");
 
 	g_object_class_install_property
 		(gobject_class, PROP_UPDATE_POLICY,
